@@ -1,16 +1,30 @@
 #coding:utf-8
 from sqlbean.db.query import Query, escape
-
 from sqlbean.db.mc_connection import mc
 from marshal import dumps, loads
-from sqlbean.metamodel import cache, lower_name, ModelBase
+from sqlbean.metamodel import cache, lower_name, ModelBase, get_or_create
 from array import array
+
+def model_dumps(self):
+    key = self.Meta.mc_key%self.id
+    value = []
+    for i in self._fields:
+        value.append(self.__dict__.get(i, None))
+    value = dumps(tuple(value))
+    return value 
+
 class McModel(object):
+    @classmethod
+    def _loads(cls, value):
+        value = loads(value)
+        value = cls(*value)
+        value._new_record = False
+        return value
 
     __metaclass__ = ModelBase
 
     debug = False
-
+    get_or_create = get_or_create
     def __init__(self, *args, **kwargs):
         'Allows setting of fields using kwargs'
         self.__dict__[self.Meta.pk] = None
@@ -24,29 +38,25 @@ class McModel(object):
     @classmethod
     def mc_get(cls, id):
         key = cls.Meta.mc_key%id
-        value = mc.get(key)
+        value = mc.get_marshal(key, cls._loads)
         if value is None:
             value = cls.get(id)
             if value:
                 value.mc_set()
-        else:
-            value = loads(value)
-            value = cls(*value)
-            value._new_record = False
         return value
 
     @classmethod
     def mc_flush_multi(cls, id_list):
         mc_key = cls.Meta.mc_key
-        result = mc.get_multi([mc_key%i for i in id_list])
+        result = mc.get_multi_marshal([mc_key%i for i in id_list], cls._loads)
         return result
 
     @classmethod
     def mc_get_multi(cls, id_list):
-        if type(id_list) not in (array,list,tuple,dict):
+        if type(id_list) not in (array, list, tuple, dict):
             id_list = tuple(id_list)
         mc_key = cls.Meta.mc_key
-        result = mc.get_multi([mc_key%i for i in id_list])
+        result = mc.get_multi_marshal([mc_key%i for i in id_list], cls._loads)
         r = {}
         for i in id_list:
             t = result.get(mc_key%i)
@@ -65,7 +75,7 @@ class McModel(object):
     def mc_get_list(cls, id_list):
         id_list = tuple(id_list)
         mc_key = cls.Meta.mc_key
-        result = mc.get_multi(mc_key%i for i in id_list)
+        result = mc.get_multi_marshal([mc_key%i for i in id_list],cls._loads)
         r = []
         for i in id_list:
             t = result.get(mc_key%i)
@@ -90,12 +100,7 @@ class McModel(object):
         mc.delete(self.Meta.mc_key%self.id)
 
     def mc_set(self):
-        key = self.Meta.mc_key%self.id
-        value = []
-        for i in self._fields:
-            value.append(self.__dict__.get(i, None))
-        value = dumps(tuple(value))
-        mc.set(key, value)
+        mc.set_marshal(key,self,dumps=model_dumps)
 
     def __setattr__(self, name, value):
         'Records when fields have changed'
